# --------------------------------------------------------
#    mmqgis_dialogs - Dialog classes for mmqgis
#
#    begin                : 10 May 2010
#    copyright            : (c) 2009 - 2010 by Michael Minn
#    email                : See michaelminn.com
#
#   MMQGIS is free software and is offered without guarantee
#   or warranty. You can redistribute it and/or modify it 
#   under the terms of version 2 of the GNU General Public 
#   License (GPL v2) as published by the Free Software 
#   Foundation (www.gnu.org).
# --------------------------------------------------------

import csv
import os.path
import operator

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *

from mmqgis_library import *

import sys
sys.path.append(os.path.dirname(__file__) + "/forms")

# ----------------------------------------------------------
#    mmqgis_attribute_export - Export attributes to CSV file
# ----------------------------------------------------------

from mmqgis_attribute_export_form import *

class mmqgis_attribute_export_dialog(QDialog, Ui_mmqgis_attribute_export_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browseoutfile, SIGNAL("clicked()"), self.browse_outfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_attributes)
		self.set_attributes()

		self.outfilename.setText(os.getcwd() + "/attributes.csv")

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output CSV File"), 
			self.outfilename.displayText(), "*.csv")
                if newname != None:
                	self.outfilename.setText(QString(newname))

	def set_attributes(self):
		self.attributes.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.attributes.addItem(field.name())

	def run(self):
		layername = str(self.sourcelayer.currentText())
		outfilename = str(self.outfilename.displayText())
		if len(outfilename) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Export", "No output CSV file given")
			return

		# Compile a list and header of selected attributes
		attribute_names = []
		for x in range(0, self.attributes.count()):
			list_item = self.attributes.item(x)
			if list_item.isSelected():
				attribute_names.append(str(list_item.text()))

		if (len(attribute_names) <= 0):
			QMessageBox.information(self.iface.mainWindow(), \
				"Attribute Export", "No attributes selected for export")
			return

		mmqgis_export_attributes(self.iface, outfilename, layername, attribute_names)


# --------------------------------------------------------
#    mmqgis_attribute_join - Join attributes from a CSV
#                            file to a shapefile
# --------------------------------------------------------

from mmqgis_attribute_join_form import *

class mmqgis_attribute_join_dialog(QDialog, Ui_mmqgis_attribute_join_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse_infile, SIGNAL("clicked()"), self.browse_infiles)
		QObject.connect(self.browse_outfile, SIGNAL("clicked()"), self.browse_outfiles)
		QObject.connect(self.browse_notfound, SIGNAL("clicked()"), self.browse_notfoundfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.joinlayer.addItem(layer.name())

		QObject.connect(self.joinlayer, SIGNAL("currentIndexChanged(QString)"), self.set_join_attributes)
		self.set_join_attributes()

		self.outfilename.setText(os.getcwd() + "/joined.shp")
		self.notfoundfilename.setText(os.getcwd() + "/notfound.csv")

        def browse_infiles(self):
		newname = QFileDialog.getOpenFileName(None, QString.fromLocal8Bit("Input CSV File"), 
			self.infilename.displayText(), "*.csv")

                if newname:
			try:
				infile = open(newname, 'r')
			except:
				QMessageBox.information(self.iface.mainWindow(), "Input CSV File", "Failure opening " + newname)
			else:
				dialect = csv.Sniffer().sniff(infile.read(1024))
				infile.seek(0)
				reader = csv.reader(infile, dialect)
				header = reader.next()
				del reader
				del infile

				self.csvfilefield.clear()
				for field in header:
					self.csvfilefield.addItem(QString(field))

                		self.infilename.setText(QString(newname))

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.outfilename.displayText(), "*.shp")
                if newname != None:
                	self.outfilename.setText(QString(newname))

        def browse_notfoundfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Unjoined CSV File"), 
			self.notfoundfilename.displayText(), "*.csv")
                if newname != None:
                	self.notfoundfilename.setText(QString(newname))

	def set_join_attributes(self):
		self.joinlayerattribute.clear()
		layername = self.joinlayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.joinlayerattribute.addItem(field.name())

	def run(self):
		layername = str(self.joinlayer.currentText())
		joinfield = str(self.csvfilefield.currentText())
		joinattribute = str(self.joinlayerattribute.currentText())
		infilename = str(self.infilename.displayText())
		outfilename = str(self.outfilename.displayText())
		notfoundname = str(self.notfoundfilename.displayText()).strip()
		addlayer = self.addtoproject.isChecked()

		if len(infilename) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", "No input CSV file given")
			return
		if len(outfilename) <= 0:
			QMessageBox.information(self.iface.mainWindow(), "Attribute Join", "No output shapefile given")
			return

		mmqgis_attribute_join(self.iface, layername, infilename, joinfield, joinattribute, outfilename, notfoundname, addlayer)

# --------------------------------------------------------
#    mmqgis_color_map - Robust layer coloring
# --------------------------------------------------------

from mmqgis_color_map_form import *

class mmqgis_color_map_dialog(QDialog, Ui_mmqgis_color_map_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.lowcolor, SIGNAL("clicked()"), self.select_color)
		QObject.connect(self.midcolor, SIGNAL("clicked()"), self.select_color)
		QObject.connect(self.highcolor, SIGNAL("clicked()"), self.select_color)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			self.layername.addItem(layer.name())
		
		self.preset.addItem("Red - Green")
		self.preset.addItem("Green - Red")
		self.preset.addItem("Blue - Brown")
		self.preset.addItem("Brown - Blue")
		self.preset.addItem("Purple - Green")
		self.preset.addItem("Green - Purple")
		QObject.connect(self.preset, SIGNAL("currentIndexChanged(QString)"), self.set_color_preset)
		self.preset.setCurrentIndex(0)

		QObject.connect(self.layername, SIGNAL("currentIndexChanged(QString)"), self.set_band_names)
		self.set_band_names()

		QObject.connect(self.bandname, SIGNAL("currentIndexChanged(QString)"), self.set_band_range)
		self.set_band_range()

        def select_color(self):
		button = self.sender()
		button_color = button.palette().color(QPalette.Normal, QPalette.Button)
		button_color = QColorDialog.getColor(button_color)
		print button_color
		if button_color.isValid():
			button.setStyleSheet("* { background-color: " + button_color.name() + " }");

	def set_color_preset(self):
		name = str(self.preset.currentText())
		if name == "Red - Green":
			self.lowcolor.setStyleSheet("* { background-color: #800000 }");
			self.midcolor.setStyleSheet("* { background-color: #808000 }");
			self.highcolor.setStyleSheet("* { background-color: #008000 }");
		elif name == "Green - Red":
			self.highcolor.setStyleSheet("* { background-color: #800000 }");
			self.midcolor.setStyleSheet("* { background-color: #808000 }");
			self.lowcolor.setStyleSheet("* { background-color: #008000 }");
		elif name == "Blue - Brown":
			self.lowcolor.setStyleSheet("* { background-color: #6e462d }");
			self.midcolor.setStyleSheet("* { background-color: #c9c865 }");
			self.highcolor.setStyleSheet("* { background-color: #346666 }");
		elif name == "Brown - Blue":
			self.highcolor.setStyleSheet("* { background-color: #6e462d }");
			self.midcolor.setStyleSheet("* { background-color: #c9c865 }");
			self.lowcolor.setStyleSheet("* { background-color: #346666 }");
		elif name == "Purple - Green":
			self.lowcolor.setStyleSheet("* { background-color: #5f309c }");
			self.midcolor.setStyleSheet("* { background-color: #fcfbbd }");
			self.highcolor.setStyleSheet("* { background-color: #147a0b }");
		elif name == "Green - Purple":
			self.highcolor.setStyleSheet("* { background-color: #5f309c }");
			self.midcolor.setStyleSheet("* { background-color: #fcfbbd }");
			self.lowcolor.setStyleSheet("* { background-color: #147a0b }");
				
	def set_band_names(self):
		self.bandname.clear()
		layer = mmqgis_find_layer(str(self.layername.currentText()))
		if layer != None:
			if layer.type() == QgsMapLayer.RasterLayer:
				self.steps.setText("256");
				for x in range(1, layer.bandCount() + 1):
					self.bandname.addItem(layer.bandName(x))

			else:
				self.steps.setText("5");
				for index, field in layer.dataProvider().fields().iteritems():
					if ((field.type() == QVariant.Double) or
					    (field.type() == QVariant.Int) or
					    (field.type() == QVariant.UInt)):
						self.bandname.addItem(field.name())

	def set_band_range(self):
		minimum = 0
		maximum = 100
		layer = mmqgis_find_layer(str(self.layername.currentText()))
		if layer != None:
			if layer.type() == QgsMapLayer.RasterLayer:
				for index in range(1, (layer.bandCount() + 1)):
					if layer.bandName(index) == self.bandname.currentText():
						minimum = layer.minimumValue(index)
						maximum = layer.maximumValue(index)

			else: # vector
				minimum, maximum, valid = mmqgis_layer_attribute_bounds(layer, str(self.bandname.currentText()))
		
		self.lowvalue.setText(str(minimum))
		self.midvalue.setText(str((minimum + maximum) / 2))
		self.highvalue.setText(str(maximum))
						

	def run(self):
		#print self.lowvalue.text()
		#print self.midvalue.text()
		#print self.highvalue.text()
		mmqgis_set_color_map(self.iface, str(self.layername.currentText()),
			str(self.bandname.currentText()), 
			float(str(self.lowvalue.text())),
			float(str(self.midvalue.text())), 
			float(str(self.highvalue.text())),
			int(self.steps.text()),
			int(self.lowcolor.palette().color(QPalette.Normal, QPalette.Button).rgb()),
			int(self.midcolor.palette().color(QPalette.Normal, QPalette.Button).rgb()),
			int(self.highcolor.palette().color(QPalette.Normal, QPalette.Button).rgb()))


# --------------------------------------------------------
#    mmqgis_geocode_google - Geocode using Google Maps
# --------------------------------------------------------

from mmqgis_geocode_google_form import *

#pyqt4-dev-tools
#designer

class mmqgis_geocode_google_dialog(QDialog, Ui_mmqgis_geocode_google_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse_infile, SIGNAL("clicked()"), self.browse_infile_dialog)
		QObject.connect(self.browse_shapefile, SIGNAL("clicked()"), self.browse_shapefile_dialog)
		QObject.connect(self.browse_notfound, SIGNAL("clicked()"), self.browse_notfound_dialog)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

        def browse_infile_dialog(self):
		newname = QFileDialog.getOpenFileName(None, QString.fromLocal8Bit("Address CSV Input File"), 
			self.infilename.displayText(), "*.csv")

                if newname:
			try:
				infile = open(newname, 'r')
			except:
				QMessageBox.information(self.iface.mainWindow(), "CSV File", "Failure opening " + newname)
				return

			if len(newname) > 4:
				prefix = newname[:len(newname) - 4]
				self.shapefilename.setText(prefix + ".shp")
				self.notfoundfilename.setText(prefix + "_notfound.csv")
			else:
				self.shapefilename.setText(os.getcwd() + "/geocoded.shp")
				self.notfoundfilename.setText(os.getcwd() + "/notfound.csv")

			dialect = csv.Sniffer().sniff(infile.read(1024))
			infile.seek(0)
			reader = csv.reader(infile, dialect)
			header = reader.next()
			del reader

			combolist = [self.addressfield, self.cityfield, self.statefield, self.countryfield]
			for box in combolist:
				box.clear()
				box.addItem(QString("(none)"))
				box.setCurrentIndex(0)
				
			for index in range(0, len(header)):
				field = header[index]
				for box in combolist:
					box.addItem(QString(field))

				if field.lower().find("addr") >= 0:
					self.addressfield.setCurrentIndex(index + 1)
				if field.lower().find("city") >= 0:
					self.cityfield.setCurrentIndex(index + 1)
				if field.lower().find("state") >= 0:
					self.statefield.setCurrentIndex(index + 1)

                	self.infilename.setText(QString(newname))

        def browse_notfound_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Not Found List Output File"), 
			self.notfoundfilename.displayText(), "*.csv")
                if newname != None:
                	self.notfoundfilename.setText(QString(newname))

        def browse_shapefile_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.shapefilename.displayText(), "*.shp")
                if newname != None:
                	self.shapefilename.setText(QString(newname))

	def run(self):
		csvname = str(self.infilename.displayText()).strip()
		shapefilename = str(self.shapefilename.displayText())
		notfoundfile = self.notfoundfilename.displayText()

		fields = [str(self.addressfield.currentText()).strip(),
			  str(self.cityfield.currentText()).strip(),
			  str(self.statefield.currentText()).strip(),
			  str(self.countryfield.currentText()).strip()]
	
		for x in range(0, len(fields)):
			if fields[x] == "(none)":
				fields[x] = ""

		print csvname + "," + "," + shapefilename
		mmqgis_geocode_google(self.iface, csvname, shapefilename, notfoundfile, fields)

# --------------------------------------------------------
#    mmqgis_grid - Grid creation plugin
# --------------------------------------------------------

from mmqgis_grid_form import *

class mmqgis_grid_dialog(QDialog, Ui_mmqgis_grid_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		self.crs = QgsCoordinateReferenceSystem()
		self.crs.createFromProj4("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
		extent = QgsRectangle (-10.0, -10.0, 10.0, 20.0)

		if (self.iface.mapCanvas() != None) and (self.iface.mapCanvas().mapRenderer() != None):
			self.crs = self.iface.mapCanvas().mapRenderer().destinationSrs()
			extent = self.iface.mapCanvas().mapRenderer().extent()

		elif self.iface.activeLayer() != None:
			extent = self.iface.activeLayer().extent()
			self.crs = self.iface.activeLayer().srs()

		# print self.crs.toProj4()

		spacing = 1
		if extent.width() > 10000:
			spacing = 1000
		elif extent.width() > 1000:
			spacing = 100
		elif extent.width() > 100:
			spacing = 10

		self.originx.setText(str(round(extent.center().x() / spacing) * spacing))
		self.originy.setText(str(round(extent.center().y() / spacing) * spacing))
		self.width.setText(str(round(extent.width() / spacing) * spacing))
		self.height.setText(str(round(extent.height() / spacing) * spacing))
		self.hspacing.setText(str(spacing))
		self.vspacing.setText(str(spacing))

		self.filename.setText(os.getcwd() + "/grid.shp")
		

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def run(self):
		savename = str(self.filename.displayText()).strip()
		hspacing = float(self.hspacing.displayText())
		vspacing = float(self.vspacing.displayText())
		width = float(self.width.displayText())
		height = float(self.height.displayText())
		originx = float(self.originx.displayText())
		originy = float(self.originy.displayText())
		addlayer = self.addtoproject.isChecked()
		mmqgis_grid(self.iface, savename, hspacing, vspacing, width, height, originx, originy, addlayer)



# --------------------------------------------------------
#    mmqgis_gridify - Snap shape verticies to grid
# --------------------------------------------------------

from mmqgis_gridify_form import *

class mmqgis_gridify_dialog(QDialog, Ui_mmqgis_gridify_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(int)"), self.layer_changed)
		
		self.filename.setText(os.getcwd() + "/gridified.shp")
		

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def layer_changed(self):
		layer = None
		layername = str(self.sourcelayer.currentText())
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				layer = searchlayer

		extent = QgsRectangle (-10.0, -10.0, 10.0, 20.0)
		if (layer):
			extent = layer.extent()

		#if (self.iface.mapCanvas() != None) and (self.iface.mapCanvas().mapRenderer() != None):
		#	extent = self.iface.mapCanvas().mapRenderer().extent()
		#elif self.iface.activeLayer():
		#	extent = self.iface.activeLayer().extent()

		self.hspacing.setText(str(extent.width() / 200))
		self.vspacing.setText(str(extent.height() / 200))

	def run(self):
		layername = str(self.sourcelayer.currentText()).strip()

		hspacing = float(self.hspacing.displayText())
		vspacing = float(self.hspacing.displayText())
		if (hspacing <= 0) or (vspacing <= 0):
			QMessageBox.information(self.iface.mainWindow(), "Grid", 
				"Invalid grid spacing: " + str(hspacing) + "/" + str(vspacing))
			return

		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Grid", "No output filename given")
			return

		mmqgis_gridify_layer(self.iface, layername, hspacing, vspacing, savename, self.addtoproject.isChecked())


# --------------------------------------------------------
#    mmqgis_hub_distance - Create shapefile of distances
#			   from points to nearest hub
# --------------------------------------------------------

from mmqgis_hub_distance_form import *

class mmqgis_hub_distance_dialog(QDialog, Ui_mmqgis_hub_distance_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayerbox.addItem(layer.name())
				self.hubslayerbox.addItem(layer.name())

		QObject.connect(self.hubslayerbox, SIGNAL("currentIndexChanged(QString)"), self.set_name_attributes)

		self.set_name_attributes()

		self.outputtype.addItem("Point")
		self.outputtype.addItem("Line to Hub")

		self.filename.setText(os.getcwd() + "/distances.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_name_attributes(self):
		self.nameattributebox.clear()
		layername = self.hubslayerbox.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.nameattributebox.addItem(field.name())

	def run(self):
		sourcename = str(self.sourcelayerbox.currentText())
		destname = str(self.hubslayerbox.currentText())
		nameattributename = str(self.nameattributebox.currentText())
		addlines = (self.outputtype.currentText() == "Line to Hub")
		addlayer = self.addtoproject.isChecked()
		savename = str(self.filename.displayText()).strip()
			
		mmqgis_hub_distance(self.iface, sourcename, destname, nameattributename, addlines, addlayer, savename)


# --------------------------------------------------------
#    mmqgis_label - Create single label points for
#		    single- or multi-feature items
# --------------------------------------------------------

from mmqgis_label_form import *

class mmqgis_label_dialog(QDialog, Ui_mmqgis_label_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_label_attributes)

		self.set_label_attributes()

		self.filename.setText(os.getcwd() + "/label.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_label_attributes(self):
		self.labelattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.labelattribute.addItem(field.name())

	def run(self):
		savename = str(self.filename.displayText()).strip()
		layername = str(self.sourcelayer.currentText())
		labelattributename = self.labelattribute.currentText()
		mmqgis_label_point(self.iface, layername, labelattributename, savename)


# --------------------------------------------------------
#    mmqgis_merge - Merge layers to single shapefile
# --------------------------------------------------------

from mmqgis_merge_form import *

class mmqgis_merge_dialog(QDialog, Ui_mmqgis_merge_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browseoutfile, SIGNAL("clicked()"), self.browse_outfiles)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		self.sourcelayers.clear()
		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayers.addItem(QString(layer.name()))
				self.sourcelayers.item(self.sourcelayers.count() - 1).setSelected(1)

		self.outfilename.setText(os.getcwd() + "/merged.shp")

        def browse_outfiles(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.outfilename.displayText(), "*.shp")
                if newname != None:
                	self.outfilename.setText(QString(newname))

	def run(self):
		layernames = []
		for x in range(0, self.sourcelayers.count()):
			if self.sourcelayers.item(x).isSelected():
				layernames.append(str(self.sourcelayers.item(x).text()))

		savename = str(self.outfilename.displayText()).strip()

		mmqgis_merge(self.iface, layernames, savename)


# ----------------------------------------------------------
#    mmqgis_select - Select features by attribute
# ----------------------------------------------------------

from mmqgis_select_form import *

class mmqgis_select_dialog(QDialog, Ui_mmqgis_select_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_select_attributes)
		self.set_select_attributes()

		comparisons = ['==', '!=', '>', '>=', '<', '<=', 'begins with', 'contains']
		for x in comparisons:
			self.comparison.addItem(x)

		self.filename.setText(os.getcwd() + "/temp.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_select_attributes(self):
		self.selectattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if selectlayer.name() == layername:
				for index, field in selectlayer.dataProvider().fields().iteritems():
					self.selectattribute.addItem(field.name())

	def run(self):
		savename = str(self.filename.displayText()).strip()
		layername = str(self.sourcelayer.currentText())
		comparisonname = self.comparison.currentText()
		comparisonvalue = str(self.value.displayText()).strip()
		selectattributename = self.selectattribute.currentText()
		addlayer = self.addtoproject.isChecked()
		mmqgis_select(self.iface, layername, selectattributename, comparisonvalue, comparisonname, savename, addlayer)


# --------------------------------------------------------
#    mmqgis_sort - Sort shapefile by attribute
# --------------------------------------------------------

from mmqgis_sort_form import *

class mmqgis_sort_dialog(QDialog, Ui_mmqgis_sort_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())
		#QObject.connect(self.sourcelayer, SIGNAL("editTextChanged(QString)"), self.set_sort_attributes)
		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_sort_attributes)

		self.set_sort_attributes()

		self.filename.setText(os.getcwd() + "/sorted.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_sort_attributes(self):
		self.sortattribute.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.sortattribute.addItem(field.name())

	def run(self):
		savename = str(self.filename.displayText()).strip()
		addlayer = self.addtoproject.isChecked()
		layername = str(self.sourcelayer.currentText())
		sortattributename = self.sortattribute.currentText()
		mmqgis_sort(self.iface, layername, sortattributename, savename, addlayer)


# --------------------------------------------------------
#    mmqgis_geocode_street_layer - Geocode addresses from street 
#			     address finder shapefile
# --------------------------------------------------------

from mmqgis_geocode_street_layer_form import *

class mmqgis_geocode_street_layer_dialog(QDialog, Ui_mmqgis_geocode_street_layer_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse_infile, SIGNAL("clicked()"), self.browse_infile_dialog)
		QObject.connect(self.browse_shapefile, SIGNAL("clicked()"), self.browse_shapefile_dialog)
		QObject.connect(self.browse_notfound, SIGNAL("clicked()"), self.browse_notfound_dialog)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.streetlayer.addItem(layer.name())

		QObject.connect(self.streetlayer, SIGNAL("currentIndexChanged(int)"), self.set_layer_attributes)
		#QObject.connect(self.streetlayer, SIGNAL("activated(int)"), self.set_layer_attributes)

		self.set_layer_attributes(0)

		self.shapefilename.setText(os.getcwd() + "/geocoded.shp")
		self.notfoundfilename.setText(os.getcwd() + "/notfound.csv")

        def browse_infile_dialog(self):
		newname = QFileDialog.getOpenFileName(None, QString.fromLocal8Bit("Address CSV Input File"), 
			self.infilename.displayText(), "*.csv")

                if newname:
			try:
				infile = open(newname, 'r')
			except:
				QMessageBox.information(self.iface.mainWindow(), "CSV File", "Failure opening " + newname)
				return
			dialect = csv.Sniffer().sniff(infile.read(1024))
			infile.seek(0)
			reader = csv.reader(infile, dialect)
			header = reader.next()
			del reader

			address_index = -1
			self.addressfield.clear()
			for field in header:
				self.addressfield.addItem(QString(field))
				if field.strip().lower() == "address":
					address_index = self.addressfield.count() - 1

			if address_index >= 0:
				self.addressfield.setCurrentIndex(address_index)

                	self.infilename.setText(QString(newname))

        def browse_notfound_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Not Found List Output File"), 
			self.notfoundfilename.displayText(), "*.csv")
                if newname != None:
                	self.notfoundfilename.setText(QString(newname))

        def browse_shapefile_dialog(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.shapefilename.displayText(), "*.shp")
                if newname != None:
                	self.shapefilename.setText(QString(newname))

	def set_layer_attributes(self, index):
		layer = None
		layername = str(self.streetlayer.currentText())
		for name, streetlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if streetlayer.name() == layername:
				layer = streetlayer

		if not layer:
			print "Layer not found " + layername
			return

		if (layer.dataProvider().crs().mapUnits() == QGis.Feet):
                	self.setback.setText("60")
		elif (layer.dataProvider().crs().mapUnits() == QGis.Meters):
                	self.setback.setText("4")
		elif (layer.dataProvider().crs().mapUnits() == QGis.Degrees):
			meters = 4 * cos(layer.extent().center().y() * pi / 180) * 6378137.0
                	self.setback.setText(str(meters))
		else:
                	self.setback.setText("4")

		self.streetname.clear()
		self.fromx.clear()
		self.fromy.clear()
		self.tox.clear()
		self.toy.clear()
		self.leftfrom.clear()
		self.leftto.clear()
		self.rightfrom.clear()
		self.rightto.clear()

		for index, field in layer.dataProvider().fields().iteritems():
			self.streetname.addItem(str(field.name()))
			self.fromx.addItem(str(field.name()))
			self.fromy.addItem(str(field.name()))
			self.tox.addItem(str(field.name()))
			self.toy.addItem(str(field.name()))
			self.leftfrom.addItem(str(field.name()))
			self.leftto.addItem(str(field.name()))
			self.rightfrom.addItem(str(field.name()))
			self.rightto.addItem(str(field.name()))

		for index, field in layer.dataProvider().fields().iteritems():
			if str(field.name()).lower() == "street":
				self.streetname.setCurrentIndex(index)
			elif str(field.name()).lower() == "xfrom":
				self.fromx.setCurrentIndex(index)
			elif str(field.name()).lower() == "yfrom":
				self.fromy.setCurrentIndex(index)
			elif str(field.name()).lower() == "xto":
				self.tox.setCurrentIndex(index)
			elif str(field.name()).lower() == "yto":
				self.toy.setCurrentIndex(index)
			elif str(field.name()).lower() == "fromleft":
				self.leftfrom.setCurrentIndex(index)
			elif str(field.name()).lower() == "fromright":
				self.rightfrom.setCurrentIndex(index)
			elif str(field.name()).lower() == "toleft":
				self.leftto.setCurrentIndex(index)
			elif str(field.name()).lower() == "toright":
				self.rightto.setCurrentIndex(index)

	def run(self):
		layername = str(self.streetlayer.currentText())
		csvname = str(self.infilename.displayText()).strip()
		addressfield = str(self.addressfield.currentText()).strip()
		shapefilename = str(self.shapefilename.displayText())
		streetname = str(self.streetname.currentText())
		fromx = str(self.fromx.currentText())
		fromy = str(self.fromy.currentText())
		tox = str(self.tox.currentText())
		toy = str(self.toy.currentText())
		leftfrom = str(self.leftfrom.currentText())
		rightfrom = str(self.rightfrom.currentText())
		leftto = str(self.leftto.currentText())
		rightto = str(self.rightto.currentText())
		setback = float(self.setback.displayText())
		notfoundfile = self.notfoundfilename.displayText()
		addlayer = self.addtoproject.isChecked()

		mmqgis_geocode_street_layer(self.iface, layername, csvname, addressfield, shapefilename, 
			streetname, fromx, fromy, tox, toy, leftfrom, rightfrom, leftto, rightto, 
			setback, notfoundfile, addlayer)

# ---------------------------------------------------------
#    mmqgis_text_to_float - Change text fields to numbers
# ---------------------------------------------------------

from mmqgis_text_to_float_form import *

class mmqgis_text_to_float_dialog(QDialog, Ui_mmqgis_text_to_float_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		QObject.connect(self.sourcelayer, SIGNAL("currentIndexChanged(QString)"), self.set_fieldnames)

		self.set_fieldnames()

		self.filename.setText(os.getcwd() + "/numeric.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def set_fieldnames(self):
		self.fieldnames.clear()
		layername = self.sourcelayer.currentText()
		for name, searchlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if searchlayer.name() == layername:
				for index, field in searchlayer.dataProvider().fields().iteritems():
					self.fieldnames.addItem(field.name())

				for index, field in searchlayer.dataProvider().fields().iteritems():
					if (field.type() == QVariant.String):
						self.fieldnames.item(index).setSelected(1)

	def run(self):
		layername = str(self.sourcelayer.currentText())
		savename = str(self.filename.displayText()).strip()
		addlayer = self.addtoproject.isChecked()

		attributes = []
		for x in range(0, self.fieldnames.count()):
			if self.fieldnames.item(x).isSelected():
				attributes.append(self.fieldnames.item(x).text())

		mmqgis_text_to_float(self.iface, layername, attributes, savename, addlayer)


# --------------------------------------------------------
#    mmqgis_voronoi - Voronoi diagram creation
# --------------------------------------------------------

from mmqgis_voronoi_form import *

class mmqgis_voronoi_dialog(QDialog, Ui_mmqgis_voronoi_form):
	def __init__(self, iface):
		QDialog.__init__(self)
		self.iface = iface
		self.setupUi(self)
		QObject.connect(self.browse, SIGNAL("clicked()"), self.browse_outfile)
        	QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.run)

		for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				self.sourcelayer.addItem(layer.name())

		self.filename.setText(os.getcwd() + "/voronoi.shp")

        def browse_outfile(self):
		newname = QFileDialog.getSaveFileName(None, QString.fromLocal8Bit("Output Shapefile"), 
			self.filename.displayText(), "*.shp")
                if newname != None:
                	self.filename.setText(QString(newname))

	def run(self):
		savename = str(self.filename.displayText()).strip()
		if savename == "":
			QMessageBox.information(self.iface.mainWindow(), "Grid", "No output filename given")
			return

		mmqgis_voronoi_diagram(self.iface,
			str(self.sourcelayer.currentText()), 
			savename, 
			self.addtoproject.isChecked())


